@@ -16,11 +16,13 @@
 # You should have received a copy of the GNU General Public License
 # along with RestAuth.  If not, see <http://www.gnu.org/licenses/>.
 
+from __future__ import unicode_literals
+
 from argparse import ArgumentParser
 
-from RestAuth.common.cli.parsers import pwd_parser
-from RestAuth.common.cli.actions import ServiceAction
-from RestAuth.Services.cli.actions import PermissionParser
+from common.cli.parsers import pwd_parser
+from common.cli.actions import ServiceAction
+from Services.cli.actions import PermissionParser
 
 # reused positional arguments:
 service_arg_parser = ArgumentParser(add_help=False)
