@@ -1,24 +1,32 @@
-"""
-This file demonstrates writing tests using the unittest module. These will pass
-when you run "manage.py test".
-
-Replace this with more appropriate tests for your application.
-"""
+# -*- coding: utf-8 -*-
+#
+# This file is part of RestAuth (https://restauth.net).
+#
+# RestAuth is free software: you can redistribute it and/or modify it under the terms of the GNU
+# General Public License as published by the Free Software Foundation, either version 3 of the
+# License, or (at your option) any later version.
+#
+# RestAuth is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without
+# even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
+# General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License along with RestAuth. If not,
+# see <http://www.gnu.org/licenses/>.
 
 from __future__ import unicode_literals
 
 from django.utils.six.moves import http_client
 
-from RestAuth.common.testdata import groupname1
-from RestAuth.common.testdata import group_backend
-from RestAuth.common.testdata import password1
-from RestAuth.common.testdata import propkey1
-from RestAuth.common.testdata import propval1
-from RestAuth.common.testdata import propval2
-from RestAuth.common.testdata import property_backend
-from RestAuth.common.testdata import RestAuthTransactionTest
-from RestAuth.common.testdata import user_backend
-from RestAuth.common.testdata import username1
+from common.testdata import groupname1
+from common.testdata import group_backend
+from common.testdata import password1
+from common.testdata import propkey1
+from common.testdata import propval1
+from common.testdata import propval2
+from common.testdata import property_backend
+from common.testdata import RestAuthTransactionTest
+from common.testdata import user_backend
+from common.testdata import username1
 
 
 class CreateUserTest(RestAuthTransactionTest):
@@ -28,20 +36,17 @@ class CreateUserTest(RestAuthTransactionTest):
         self.assertEqual(list(user_backend.list()), [])
 
     def test_dry_run_create_user_with_pass(self):
-        resp = self.post(
-            '/test/users/', {'user': username1, 'password': password1})
+        resp = self.post('/test/users/', {'user': username1, 'password': password1})
         self.assertEqual(resp.status_code, http_client.CREATED)
         self.assertEqual(list(user_backend.list()), [])
 
     def test_dry_run_create_user_with_props(self):
-        resp = self.post('/test/users/', {'user': username1,
-                                          'properties': {'foo': 'bar'}})
+        resp = self.post('/test/users/', {'user': username1, 'properties': {'foo': 'bar'}})
         self.assertEqual(resp.status_code, http_client.CREATED)
         self.assertEqual(list(user_backend.list()), [])
 
     def test_dry_run_create_user_with_pass_and_props(self):
-        content = {'user': username1, 'password': password1,
-                   'properties': {'foo': 'bar'}}
+        content = {'user': username1, 'password': password1, 'properties': {'foo': 'bar'}}
         resp = self.post('/test/users/', content)
         self.assertEqual(resp.status_code, http_client.CREATED)
         self.assertEqual(list(user_backend.list()), [])
@@ -97,9 +102,12 @@ class CreatePropertyTest(RestAuthTransactionTest):
 
     def test_create_invalid_property(self):
         url = '/test/users/%s/props/' % self.user.username
-        resp = self.post(url, {'prop': propkey1, 'value': 'foo/bar'})
-        self.assertEqual(resp.status_code, http_client.CREATED)
+        resp = self.post(url, {'prop': 'foo/bar', 'value': propval1})
+        self.assertEqual(resp.status_code, http_client.PRECONDITION_FAILED)
+        self.assertProperties(self.user, {})
 
+        resp = self.post(url, {'prop': 'foo:bar', 'value': propval1})
+        self.assertEqual(resp.status_code, http_client.PRECONDITION_FAILED)
         self.assertProperties(self.user, {})
 
     def test_create_property_for_non_existing_user(self):
