@@ -2,22 +2,18 @@
 #
 # This file is part of RestAuth (https://restauth.net).
 #
-# RestAuth is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
+# RestAuth is free software: you can redistribute it and/or modify it under the terms of the GNU
+# General Public License as published by the Free Software Foundation, either version 3 of the
+# License, or (at your option) any later version.
 #
-# RestAuth is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
+# RestAuth is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without
+# even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+# General Public License for more details.
 #
-# You should have received a copy of the GNU General Public License
-# along with RestAuth.  If not, see <http://www.gnu.org/licenses/>.
+# You should have received a copy of the GNU General Public License along with RestAuth.  If not,
+# see <http://www.gnu.org/licenses/>.
 
-"""
-This module implements all HTTP queries to ``/user/*``.
-"""
+"""This module implements all HTTP queries to ``/user/*``."""
 
 from __future__ import unicode_literals
 
@@ -30,18 +26,18 @@ from django.utils import six
 from RestAuthCommon import resource_validator
 from RestAuthCommon.error import PreconditionFailed
 
-from RestAuth.Users.validators import validate_username
-from RestAuth.backends import property_backend
-from RestAuth.backends import user_backend
-from RestAuth.common.errors import PasswordInvalid
-from RestAuth.common.errors import UserNotFound
-from RestAuth.common.responses import HttpResponseCreated
-from RestAuth.common.responses import HttpResponseNoContent
-from RestAuth.common.responses import HttpRestAuthResponse
-from RestAuth.common.types import parse_dict
-from RestAuth.common.views import RestAuthResourceView
-from RestAuth.common.views import RestAuthSubResourceView
-from RestAuth.common.views import RestAuthView
+from Users.validators import validate_username
+from backends import property_backend
+from backends import user_backend
+from common.errors import PasswordInvalid
+from common.errors import UserNotFound
+from common.responses import HttpResponseCreated
+from common.responses import HttpResponseNoContent
+from common.responses import HttpRestAuthResponse
+from common.types import parse_dict
+from common.views import RestAuthResourceView
+from common.views import RestAuthSubResourceView
+from common.views import RestAuthView
 
 
 class UsersView(RestAuthView):
@@ -53,32 +49,32 @@ class UsersView(RestAuthView):
 
     post_format = {
         'mandatory': (('user', six.string_types),),
-        'optional': (('password', six.string_types),
-                     ('properties', dict),
-                    )
+        'optional': (
+            ('password', six.string_types),
+            ('properties', dict),
+        ),
     }
     post_required = (('user', six.string_types),)
     post_optional = (('password', six.string_types),
                      ('properties', dict))
 
     def get(self, request, largs, *args, **kwargs):
-        """
-        Get all users.
-        """
+        """Get all users."""
+
         if not request.user.has_perm('Users.users_list'):
             return HttpResponseForbidden()
 
-        names = user_backend.list()
+        names = [n.lower() for n in user_backend.list()]
         return HttpRestAuthResponse(request, names)
 
     def post(self, request, largs, dry=False):
-        """
-        Create a new user.
-        """
+        """Create a new user."""
+
         if not request.user.has_perm('Users.user_create'):
             return HttpResponseForbidden()
 
         name, password, properties = self._parse_post(request)
+        name = name.lower()
 
         # check username:
         if not resource_validator(name):
@@ -95,15 +91,13 @@ class UsersView(RestAuthView):
         if properties is not None:
             for key in six.iterkeys(properties):
                 if not resource_validator(key):
-                    raise PreconditionFailed(
-                        "Property contains invalid characters")
+                    raise PreconditionFailed("Property contains invalid characters")
 
         # If ResourceExists: 409 Conflict
         # If PasswordInvalid: 412 Precondition Failed
-        user = user_backend.create(username=name, password=password,
-                                   properties=properties,
-                                   property_backend=property_backend,
-                                   dry=dry)
+        user = user_backend.create(
+            username=name, password=password, properties=properties,
+            property_backend=property_backend, dry=dry)
 
         self.log.info('%s: Created user', user.username, extra=largs)
         return HttpResponseCreated(request, 'users.user', name=user.username)
@@ -211,13 +205,10 @@ class UserPropsIndex(RestAuthResourceView):
         user = user_backend.get(username=name)
 
         # If PropertyExists: 409 Conflict
-        key, value = property_backend.create(user=user, key=key,
-                                             value=value, dry=dry)
+        key, value = property_backend.create(user=user, key=key, value=value, dry=dry)
 
-        self.log.info(
-            'Created property "%s" as "%s"', key, value, extra=largs)
-        return HttpResponseCreated(request, 'users.user.props.prop',
-                                   name=name, subname=key)
+        self.log.info('Created property "%s" as "%s"', key, value, extra=largs)
+        return HttpResponseCreated(request, 'users.user.props.prop', name=name, subname=key)
 
     def put(self, request, largs, name):
         """
@@ -231,11 +222,9 @@ class UserPropsIndex(RestAuthResourceView):
         properties = parse_dict(request)
         for key in six.iterkeys(properties):
             if not resource_validator(key):
-                raise PreconditionFailed(
-                    "Property contains invalid characters")
+                raise PreconditionFailed("Property contains invalid characters")
 
-        property_backend.set_multiple(user=user,
-                                      props=properties)
+        property_backend.set_multiple(user=user, props=properties)
         return HttpResponseNoContent()
 
 
@@ -275,16 +264,13 @@ class UserPropHandler(RestAuthSubResourceView):
         # If UserNotFound: 404 Not Found
         user = user_backend.get(username=name)
 
-        key, old_value = property_backend.set(user=user, key=subname,
-                                              value=value)
+        key, old_value = property_backend.set(user=user, key=subname, value=value)
 
         if old_value is None:  # new property
             self.log.info('Set to "%s"', value, extra=largs)
-            return HttpResponseCreated(request, 'users.user.props.prop',
-                                       name=name, subname=key)
+            return HttpResponseCreated(request, 'users.user.props.prop', name=name, subname=key)
         else:  # existing property
-            self.log.info('Changed from "%s" to "%s"',
-                          old_value, value, extra=largs)
+            self.log.info('Changed from "%s" to "%s"', old_value, value, extra=largs)
             return HttpRestAuthResponse(request, old_value)
 
     def delete(self, request, largs, name, subname):
